@@ -1,8 +1,43 @@
 from typing import Dict, Union, Optional, Tuple
 from threading import Thread, Event
+from subprocess import Popen, PIPE
+import shlex
+import logging
+
 import requests
 
 
+def check_host(host: str, port: int = 22, logger_name: str = "") -> bool:
+    """Check if a given host:port combination is online
+
+    Uses :code:`nc` process to perform the check.
+
+    Args:
+        host: name or addr of host
+        port: port to check. Defaults to 22
+
+    """
+    if "@" in host:
+        host = host.split("@")[1]
+    if logger_name:
+        logger = logging.getLogger(logger_name)
+    p = Popen(shlex.split(f"nc -z -v {host} {port}"), stdout=PIPE, stderr=PIPE)
+    out, err = p.communicate(timeout=1)
+    output = err.decode("utf-8").lower()
+    if "sent" in output and "received" in output:
+        return True
+    elif "no route" in output:
+        return False
+    elif "connection refused" in output:
+        return False
+    else:
+        if logger_name:
+            logger.error(f"Unknown output {output}")
+        else:
+            print(f"Unknown output {output}")
+        return False
+
+
 class Get:
     """A :code:`get` method with progress tracking.
 
@@ -22,12 +57,14 @@ class Get:
     """
 
     def __init__(self):
+        self._init: Dict[str, str] = {}
         self._status: Dict[str, int] = {}
         self._threads: Dict[str, Thread] = {}
         self._aborted: Dict[str, Event] = {}
         self._finished: Dict[str, Event] = {}
         self._dl_bytes: Dict[str, int] = {}
         self._progress: Dict[str, Union[int, float]] = {}
+        self._responses: Dict[str, requests.Response] = {}
         self._result: Dict[str, bytes] = {}
 
     def __call__(self, url: str, **kwargs):
@@ -45,8 +82,10 @@ class Get:
         self._threads[url].start()
 
     def _call_subr(self, url, **kwargs):
+        self._init[url] = True
         content = bytearray()
         response = requests.get(url, stream=True, **kwargs)
+        self._responses[url] = response
         self._status[url] = response.status_code
         if response.status_code != 200:
             self._finished[url].set()
@@ -67,7 +106,7 @@ class Get:
                 else:
                     break
             self._finished[url].set()
-            if self._aborted[url]:
+            if self._aborted[url].is_set():
                 self._result[url] = None
             else:
                 self._result[url] = content
@@ -76,21 +115,29 @@ class Get:
         """Abort the operation for `url`."""
         self._aborted[url].set()
 
+    def cancel(self, url: str) -> bool:
+        """Same as :meth:`abort`"""
+        return self.abort(url)
+
     def aborted(self, url: str) -> bool:
         """Check if the operation for `url` was aborted."""
         return self._aborted[url].is_set()
 
-    def result(self, url: str) -> Tuple[int, Union[bytes, bytearray]]:
+    def cancelled(self, url: str) -> bool:
+        """Same as aborted."""
+        return self.aborted(url)
+
+    def result(self, url: str) -> Tuple[int, Union[bytes, bytearray], requests.Response]:
         """Return the result of the fetch operation.
 
         Args:
             url: The url for which to perform operation
 
         Returns:
-            A tuple of status and response content.
+            A tuple of status, response and response content.
 
         """
-        return self._status[url], self._result[url]
+        return self._status[url], self._responses[url], self._result[url]
 
     def finished(self, url: str) -> Optional[bool]:
         """Check if the URL has been fetched.
@@ -114,6 +161,15 @@ class Get:
         """
         return self._dl_bytes.get(url)
 
+    def url_init(self, url: str) -> bool:
+        """Check if URL fetch has started.
+
+        Args:
+            url: The url for which to perform operation
+
+        """
+        return bool(self._init.get(url, False))
+
     def progress(self, url: str) -> Optional[float]:
         """Display progress of URL.
 
